#pragma once
#include <windows.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


//
// LineDef attributes.
//

// Solid, is an obstacle.
#define ML_BLOCKING		1

// Blocks monsters only.
#define ML_BLOCKMONSTERS	2

// Backside will not be present at all
//  if not two sided.
#define ML_TWOSIDED		4

// If a texture is pegged, the texture will have
// the end exposed to air held constant at the
// top or bottom of the texture (stairs or pulled
// down things) and will move with a height change
// of one of the neighbor sectors.
// Unpegged textures allways have the first row of
// the texture at the top pixel of the line for both
// top and bottom textures (use next to windows).

// upper texture unpegged
#define ML_DONTPEGTOP		8

// lower texture unpegged
#define ML_DONTPEGBOTTOM	16	

// In AutoMap: don't map as two sided: IT'S A SECRET!
#define ML_SECRET		32

// Sound rendering: don't let sound cross two of these.
#define ML_SOUNDBLOCK		64

// Don't draw on the automap at all.
#define ML_DONTDRAW		128

// Set if already seen, thus drawn in automap.
#define ML_MAPPED		256


#pragma pack(1)
typedef struct
{
	short start_vert;
	short end_vert;
	short flags;
	short special;
	short sector;
	short right_sidedef;
	short left_sidedef;
} linedef_t;
#pragma pack(8)

#pragma pack(1)
typedef struct
{
	short x;
	short y;
	short angle;
	short type;
	short flags;
} thing_t;
#pragma pack(8)

#pragma pack(1)
typedef struct
{
	short x;
	short y;
} vert_t;
#pragma pack(8)

#pragma pack(1)
typedef struct
{
	short tex_xoffset;
	short tex_yoffset;
	char tex_upper[8];
	char tex_lower[8];
	char tex_middle[8];
	// front sector towards viewer
	short sector;
} sidedef_t;
#pragma pack(8)

#pragma pack(1)
// Segments/Cells enclosed area by lines may not be convex
typedef struct
{
	short floor_height;
	short ceil_height;
	char tex_floor[8];
	char tex_ceil[8];
	short light_level;
	short type;
	short tag;
} sector_t;
#pragma pack(8)

#pragma pack(1)
// segment of a line split by partition line
typedef struct
{
	short start_vert;
	short end_vert;
	short angle;
	short linedef;
	short sidedef;
	short offset;
} seg_t;
#pragma pack(8)

#pragma pack(1)
// BSP split segments (cells) always convex
typedef struct
{
	short num_seg;
	short first_seg;
} subsector_t;
#pragma pack(8)

#pragma pack(1)
typedef struct
{
	// Partition line from (x,y) to (x+dx,y+dy)
	short x;
	short y;
	short dx;
	short dy;
	short bbox[2][4]; // right then left
	short child[2]; // right then left
} node_t;
#pragma pack(8)

typedef struct
{
	vert_t *pvert;
	int num_vert;
	linedef_t *pline;
	int num_line;
	thing_t *pthing;
	int num_thing;
	sidedef_t *pside;
	int num_side;
	sector_t *psector;
	int num_sector;
	seg_t *pseg;
	int num_seg;
	subsector_t *psubsector;
	int num_subsector;
	node_t *pnode;
	int num_node;
	// lazy hacks
	HDC hdc;
	HPEN red, green[255], green_dash[255], blue;
	float scale;
	float offset_x;
	float offset_y;
	int max_depth;
	int max_draw;
	HBRUSH cross_brush;
	bool draw_order;
} waddata_t;



char *get_file(char *filename, unsigned int *size);
void *get_wadfile(char *wadfile, char *level, char *lump, int *lump_size, char **pdata);